
ARFoundation = {
    isInit = false,
    isStart = false,
    game = nil
}

ARFoundation["ARConfig"] = {
    WorldTracking = 0,
    VPlaneDetect = 1,
    HPlaneDetect = 2,
    NormalCamera = 3,
    
}

ARFoundation["ARState"] = { 
    NOT_AVAILABLE = 0,
    NORMAL = 1 
}

ARFoundation["RegisterType"] = {
    ADD_PLANE = 0,
    UPDATE_PLANE = 1,
    DELETE_PLANE = 2,
    UPDATE_FRAME = 3
}

function ARLog(type, msg)
    print("ARFoundation "..type.." : "..msg)
end

--初始化AR模块，初始化成员变量
function ARFoundation:Init(gameObj)
    if ARFoundation.isInit ~= true then
        print("game is :", gameObj)
        ARFoundation.game = gameObj
        ARFoundation.logicObj = gameObj.engineObj:GetLogicObjProxy()
        ARFoundation.isInit = true
        ARFoundation.isUpdateCamera = true
        ARFoundation.isStartSensor = false
        ARFoundation.isStart = false
        ARFoundation.isArStart = false
        ARFoundation.isUpdate = nil
        ARFoundation.isReady = false
        ARFoundation.bFront = false

        ARFoundation.checkARCallObj = SClientCallBackObjData()
	    ARFoundation.installARCallObj = SClientCallBackObjData()
        ARFoundation.startARCallObj = SClientCallBackObjData()
        ARFoundation.stopARCameraCallObj = SClientCallBackObjData()
        ARFoundation.stopCameraCallObj = SClientCallBackObjData()
        ARFoundation.hitTestCallObj = SClientCallBackObjData()
        ARFoundation.arInfoCallObj = SClientCallBackObjData()
        ARFoundation.permissionCallObj = SClientCallBackObjData()

        ARFoundation.isStartReturn = false
    end
end

--检测AR是否支持
function ARFoundation:CheckSupport()
    if ARFoundation.isInit ~= true then
        ARLog("Warning", "not init, can not check support")
        return false
    end
    print("start check support")
    ARFoundation.logicObj:CallClientFun("invoke.arcamera.checkSupport", {}, "CheckSupportCallBack", ARFoundation.checkARCallObj)
    print("end check support")
end

--安装AR engine
function ARFoundation:InstallAR()
    ARFoundation.logicObj:CallClientFun("invoke.arcamera.requestInstall", {}, "InstallARCallBack", ARFoundation.installARCallObj)	
end

--启动AR
function ARFoundation:Start(isUpdateCamera, isStartSensor, bFront )
    --初始化检测
    if ARFoundation.isInit ~= true then
        print("ARFoundation not init, so can not start")
        return false
    end

    --AR支持检测,相机权限是否开启
    ARLog("Info", "start check permission")
    ARFoundation.logicObj:CallClientFun("invoke.permission.checkPermission", {permission="camera"}, "CheckPermissionCallBack", ARFoundation.permissionCallObj)
    ARFoundation:CheckSupport()

    if bFront ~= nil and bFront == true then
        ARFoundation.bFront = true
    else
        ARFoundation.bFront = nil
    end

    if isUpdateCamera ~= nil then
        ARFoundation.isUpdateCamera = isUpdateCamera
    end
    if isStartSensor ~= nil then
        ARFoundation.isStartSensor = isStartSensor
    end
end

function DeviceMotionCallBack(ojb, msg)
    -- print("ARFoundation: DeviceMotionCallBack msg", msg)
end

function ARFoundation:DoStart()
    ARFoundation.isStart = true
    --调用客户端接口，启动AR模块
    ARLog("Info", "staring ar")
    startParam = {}
    if ARFoundation.isStartSensor then
        key = "invoke.camera.start"
    else
        key = "invoke.arcamera.start"
    end
    if ARFoundation.bFront ~= nil then
        key = "invoke.camera.start"
        if ARFoundation.bFront then
            startParam["front"] = "true"
        else
            startParam["front"] = "false"
        end
    end
    print("ARFoundation:DoStart key", key, ARFoundation.bFront )
    ARFoundation.isStartReturn = false
    ARFoundation.logicObj:CallClientFun(key, startParam, "StartCallBack", ARFoundation.startARCallObj)
    --判断是否要启动传感器
    if ARFoundation.isStartSensor == true then
        ARFoundation.motionCallObj = SClientCallBackObjData()
        ARFoundation.orientationCallObj = SClientCallBackObjData()
        ARFoundation.logicObj:CallClientFun("register.sensor.device_motion", {}, "DeviceMotionCallBack", ARFoundation.motionCallObj)
        ARFoundation.logicObj:CallClientFun("register.sensor.device_orientation", {}, "DeviceOrientationCallBack", ARFoundation.orientationCallObj)
    end
end

--停止AR
function ARFoundation:Stop()
    if ARFoundation.isStart ~= true then
        ARLog("Warning", "ARFoundation not start, so can not stop")
        return false
    end
    ARFoundation.logicObj:CallClientFun("invoke.arcamera.stop", {}, "StopCallBack", ARFoundation.stopARCameraCallObj)
    ARFoundation.logicObj:CallClientFun("invoke.camera.stop", {}, "StopCallBack", ARFoundation.stopCameraCallObj)
end

--校验回调消息
function ARFoundation:CheckMessage(msg)
    if msg["success"] == nil or msg["data"] == nil then
        ARLog("Error", "message formart error")
        return false
    end
    if msg["success"] == false then
        if msg["errorCode"] == nil then
            ARLog("Error", "return false")
            return false
        end
    end
    return true
end

--CheckSupport回调
function CheckSupportCallBack(obj, msg)
    print("check support callback", msg)
    ARFoundation.isCheckReturn = true
    local msg = cjson.decode(msg)
    if msg["success"] == nil or msg["data"] == nil then
        ARLog("Error", "checkSupport msg error")
        return
    end

    local ret = msg["success"]
    if ret == false then
        ARFoundation.isSupport = false
        ARFoundation.arEngineType = "unknow"
        if msg["data"]["ar_engine"] ~= nil then
            ARFoundation.arEngineType = msg["data"]["ar_engine"]
        end

        ec = msg["errorCode"]
        if ec == "-1" then
            ARLog("Info", "not support AR")
        -- elseif ec == "2" then
        --     ARLog("Info", "ar engine not install")
            -- ARFoundation.isNeedInstall = true
            -- ret = ARFoundation:InstallAR()
            -- ARFoundation.isSupport = ret
        else
            ARLog("Info", "not support AR, unknown error")
        end
    else
        ARFoundation.isSupport = true
        ARFoundation.arEngineType = "unknow"
        if msg["data"]["ar_engine"] ~= nil then
            ARFoundation.arEngineType = msg["data"]["ar_engine"]
        end
    end
end

--CheckPermission回调
function CheckPermissionCallBack(obj, msg)
    print("check permission callback", msg)
    local msg = cjson.decode(msg)
    if ARFoundation:CheckMessage(msg) ~= true then
        ARLog("Error", "check permission msg error")
        return
    end
    local ret = msg["success"]
    if ret == false then
        --没权限，申请权限
        ARFoundation.isPermission = false
        ARFoundation.logicObj:CallClientFun("invoke.permission.requestPermission", {permission="camera"}, "RequestPermissionCallBack", ARFoundation.permissionCallObj)
    else
        --有权限，直接启动
        ARLog("Info", "check permission success")
        ARFoundation.isPermission = true
        ARFoundation:DoStart()
    end
end

--RequestPermission回调
function RequestPermissionCallBack(obj, msg)
    print("request permission callback", msg)
    local msg = cjson.decode(msg)
    if ARFoundation:CheckMessage(msg) ~= true then
        ARLog("Error", "request permission msg error")
        return
    end
    local ret = msg["success"]
    if ret == true then
        ARFoundation:DoStart()
    else
        ARLog("Error", "camera permission request failed！")
        ARFoundation.requestCameraPermissionFail = true
    end
end

--InstallAR回调
function InstallARCallBack(obj, msg)
    print("install ar callback", msg)
    ARFoundation.isInstallReture = true
    local msg = cjson.decode(msg)
    if ARFoundation:CheckMessage(msg) ~= true then
        ARLog("Error", "install AR msg error")
        return
    end

    local ret = msg["success"]
    if ret == false then
        ARFoundation.isSupport = false
        ec = msg["errorCode"]
        if ec == "-1" then
            ARLog("Info", "not support install AR engine")
        elseif ec == "-2" then
            ARLog("Info", "ar engine install error")
        else
            ARLog("Info", "user cancel install")
        end
    else
        ARFoundation.logicObj:CallClientFun("invoke.permission.checkPermission", {permission="camera"}, "CheckPermissionCallBack", ARFoundation.permissionCallObj)
    end
    return ret   
end

--Start回调
function StartCallBack(obj, msg)
    print("start ar callback", msg)
    local msg = cjson.decode(msg)
    if ARFoundation:CheckMessage(msg) ~= true then
        ARLog("Error", "start AR msg error")
        return
    end
    ARFoundation.isStart = msg["success"]
    ARFoundation.isArStart = msg["success"]
    ARFoundation.isStartReturn = true
    --启动成功则注册AR信息回调
    if ARFoundation.isStart == true then
        print("register arcamera", msg)
        ARFoundation.logicObj:CallClientFun("register.arcamera.ar_info_notify", {}, "ARInfoNotify", ARFoundation.arInfoCallObj)
    end
end

--Stop回调
function StopCallBack(obj, msg)
    print("stop ar callback", msg)
    local msg = cjson.decode(msg)
    if ARFoundation:CheckMessage(msg) ~= true then
        ARLog("Error", "start AR msg error")
        return
    end
    
    local ret = msg["success"]
    if ret == true then
        ARFoundation.isStart = false
        ARFoundation.isReady = false
    end
end

--ARInfoNotify回调
function ARInfoNotify(obj, msg)
    --print("ar info callback", msg)
    local msg = cjson.decode(msg)
    if not msg["data"] or not msg["data"]["viewMatrix"] or not msg["data"]["projMatrix"] then
        print("invalid ARInfoNotify msg", msg)
        return 
    end

    local vmObj = msg["data"]["viewMatrix"]
    local pmObj = msg["data"]["projMatrix"]
    ARFoundation.isReady = msg["data"]["isTracking"]
    ARFoundation.isUpdate = msg["data"]["isUpdate"]

    local vm = ARFoundation:TableToMff4(vmObj)
    local pm = ARFoundation:TableToMff4(pmObj)

    if ARFoundation.game then
        local mViewPortMatrix = ARFoundation.game.engineObj:GetEngineContext():GetViewPortMatrix()
        pm:multMatrix(mViewPortMatrix, pm)
    end

    if ARFoundation.isStartSensor == true then
        local rm = ARFoundation.rotateMatrix
        vm = rm
    end

    if ARFoundation.isUpdateCamera == true then
        -- ARFoundation.game.scene.engineObj:GetCamera():SetViewProjMatrix(vm, pm)
        vm.m41, vm.m42, vm.m43 = vm.m41 * 100, vm.m42 * 100, vm.m43 * 100
        g_game:OnUpdateViewProj(vm, pm)
    end

    --plane 数据更新
    local plane = msg["data"]["plane"]
    if plane ~= nil then
        ARFoundation.planeDetect = {}
        for i = 1, #plane do
            local planeInfo = {}
            planeInfo.hashCode = plane[i]["hashCode"]
            local pose = ARFoundation:TableToMff4(plane[i]["planeCenterPose"])
            pose.m41, pose.m42, pose.m43 = pose.m41 * 100, pose.m42 * 100, pose.m43 * 100
            planeInfo.planeCenterPose = pose

            local center = plane[i]["planeCenter"]
            planeInfo.planeCenter = V3f(center[1] * 100, center[2] * 100, center[3] * 100)

            planeInfo.planeExtentX = plane[i]["planeExtentX"] * 100
            planeInfo.planeExtentZ = plane[i]["planeExtentZ"] * 100

            -- get plane 4 vertexs
            local dx = planeInfo.planeExtentX / 2.0
            local dz = planeInfo.planeExtentZ / 2.0
            local p1 = V4f()
            p1.x,p1.y,p1.z,p1.w = dx, 0, dz, 1
            local p2 = V4f()
            p2.x,p2.y,p2.z,p2.w = -dx, 0, dz, 1
            local p3 = V4f()
            p3.x,p3.y,p3.z,p3.w = -dx, 0, -dz, 1
            local p4 = V4f()
            p4.x,p4.y,p4.z,p4.w = dx, 0, -dz, 1

            local pp1,pp2,pp3,pp4 = V4f(), V4f(), V4f(), V4f()
            pose:multVec4Matrix(p1, pp1)
            pose:multVec4Matrix(p2, pp2)
            pose:multVec4Matrix(p3, pp3)
            pose:multVec4Matrix(p4, pp4)
            
            planeInfo.planeVertex1 = V3f(pp1.x, pp1.y, pp1.z)
            planeInfo.planeVertex2 = V3f(pp2.x, pp2.y, pp2.z)
            planeInfo.planeVertex3 = V3f(pp3.x, pp3.y, pp3.z)
            planeInfo.planeVertex4 = V3f(pp4.x, pp4.y, pp4.z)
            table.insert(ARFoundation.planeDetect, planeInfo)
        end
    end
    -- print("plane center is", ARFoundation.planeCenter)
end

--HitTest触发
function ARFoundation:HitTest(xp, yp)
    print("call arfoundation hit test")
    if ARFoundation.isStart ~= true then
        ARLog("Error", "ARFoundation not start, so can not hit test")
        return
    end
    ARFoundation.logicObj:CallClientFun("invoke.arcamera.hitTest", {x=xp, y=yp}, "HitTestCallBack", ARFoundation.hitTestCallObj)
end

--HitTest回调
function HitTestCallBack(obj, msg)
    print("ar hit test callback", msg)
    local msg = cjson.decode(msg)
    if ARFoundation:CheckMessage(msg) ~= true then
        ARLog("Error", "start AR msg error")
        return
    end

    local ret = msg["success"]
    if ret == true then
        ARFoundation.hitTestResult = msg["data"]["hit_result"]["translate"]
        ARFoundation.hitTestResult[1] = ARFoundation.hitTestResult[1] * 100
        ARFoundation.hitTestResult[2] = ARFoundation.hitTestResult[2] * 100
        ARFoundation.hitTestResult[3] = ARFoundation.hitTestResult[3] * 100
    end
end

--DeviceOrientation回调
function DeviceOrientationCallBack(obj, msg)
    -- print("DeviceOrientationCallBack, msg", msg)
    local jsonMsg = cjson.decode(msg)
    local msg = jsonMsg
    if msg and msg["data"] then
        msg = msg["data"]
    end
    if msg and msg["rotation_matrix"] then
        -- local msg = msg["data"]
        local euler = {}
        euler["alpha"] = msg["alpha"]
        euler["beta"] = msg["beta"]
        euler["gamma"] = msg["gamma"]
        local rm = msg["rotation_matrix"]
        ARFoundation.euler = euler
        local rr = ARFoundation:TableToMff4(rm)
        ARFoundation.rotateMatrix = rr

        local rotate = M44f()
        local vAxis = V3f(0.0, 0.0, 0.0)
        vAxis.x = 1.0
        vAxis.y = 0.0
        vAxis.z = 0.0
        rotate:setAxisAngle(vAxis, 3.1415926 / 2.0)
        rotate:multMatrix(rr, rotate)
        ARFoundation.rotateMatrix = rotate

        g_game:OnUpdateViewProj(rotate)
    else
        print("DeviceOrientationCallBack: invalid msg")
    end
end

function ARFoundation:GetRotateMatrix(alpha, beta, gamma)
    print("ios euler :", alpha*180/3.14, beta*180/3.14, gamma*180/3.14)
    local euler = Eulerf(beta, alpha, -gamma, 0x1001)
    local rotateMatrix = euler:toMatrix44()

    --原生IOS陀螺仪 alpha-x-pitch  gamma-y-roll beta-z-yaw   
    --https://developer.apple.com/documentation/coremotion/getting_processed_device-motion_data/understanding_reference_frames_and_device_attitude
    --[[cx = math.cos(alpha)
    cy = math.cos(gamma)
    cz = math.cos(beta)
    sx = math.sin(alpha)
    sy = math.sin(gamma)
    sz = math.sin(beta)]]--

    --W3C标准  beta-x  gamma-y alpha-z
    cx = math.cos(beta)
    cy = math.cos(gamma)
    cz = math.cos(alpha)
    sx = math.sin(beta)
    sy = math.sin(gamma)
    sz = math.sin(alpha)

    local rm = M44f()
    --YXZ  原生IOS陀螺仪数据顺规 & W3C顺规范是ZXY，但是我们需要的是YXZ（手机竖起来）
    rm.m11 = cz*cy-sz*sx*sy
    rm.m12 = -cx*sz
    rm.m13 = cz*sy+cy*sz*sx
    rm.m21 = sz*cy + cz*sx*sy
    rm.m22 = cz*cx
    rm.m23 = sz*sy-cz*cy*sx
    rm.m31 = -cx*sy
    rm.m32 = sx
    rm.m33 = cx*cy

    --选择矩阵顺序，需要和相机矩阵相反，比如手机旋转顺序的 Y-X-Z，那么物体就要反着转 Z-X-Y
    local rotate = M44f()
    local vAxis = V3f(0.0, 0.0, 0.0)
	vAxis.x = 1.0
	vAxis.y = 0.0
	vAxis.z = 0.0
    rotate:setAxisAngle(vAxis, 3.1415926 / 2.0)
    rotate:multMatrix(rm, rotate)
    rm:multMatrix(rotate, rm)
    rm = rotate

    ARFoundation.rotateMatrix = rm
end

function ARFoundation:TableToMff4(obj)
    local m = M44f()
	m.m11 = obj[1]
	m.m12 = obj[2]
	m.m13 = obj[3]
	m.m14 = obj[4]
    m.m21 = obj[5]
	m.m22 = obj[6]
	m.m23 = obj[7]
	m.m24 = obj[8]
	m.m31 = obj[9]
	m.m32 = obj[10]
	m.m33 = obj[11]
	m.m34 = obj[12]
	m.m41 = obj[13]
	m.m42 = obj[14]
	m.m43 = obj[15]
    m.m44 = obj[16]
    return m
end