local TouchGestureEngine = require "touch_gesture_engine/touch_gesture_engine"

GameObj = {}
Game = {}
g_game = nil

function Game:new(engineObj)
    o = {}
    setmetatable(o, self)
    self.__index = self
    o.engineObj = engineObj
    o.objTable = {}
    o.isEditor = false
    o.recorderInitCallObj = SClientCallBackObjData()
    o.recorderStartCallObj = SClientCallBackObjData()
    o.recorderStopCallObj = SClientCallBackObjData()
    o.recordCompleteCallObj = SClientCallBackObjData()
    o.recordProcessCallObj = SClientCallBackObjData()
    o.recordPhotoAlbumCallObj = SClientCallBackObjData()
    o.recordPhotoReleaseCallObj = SClientCallBackObjData()
    o.soDownloadCallback = SClientCallBackObjData()
    o.downloadStateCallObj = SClientCallBackObjData()
    o.downloadsoCallObj = SClientCallBackObjData()

    if engineObj then
        o.render = engineObj:GetEngineContext():GetRenderer()
        o.context = engineObj:GetEngineContext()
        o.isEditor = ( engineObj:GetEngineContext():GetEvnVar("IsEditor") == "1" )
    end

    o.recordWidth = 720
    o.recordHeight = 1080
    o.bStartRecordSuccess = false
    o.recordIsRead = false
    o.recordFileCache = {}

    if engineObj then
        local platform = o.context:GetEvnVar("p_fr")
        o.bIsIos = true
        print("platform is ", platform)
        if platform == "android" then
            o.bIsIos = false
        else
            o.recordIsRead = true
        end
    end
    return o
end

function Game:onDetector(time, frameBuffer)
    return frameBuffer
end

function Game:gameRender(time, frameBuffer)
    return self:onRender(time, self:onDetector(time, frameBuffer))
end

function Game:onRender(time, frameBuffer)
    return frameBuffer
end

function initRecordCallback(obj, msg)
    print("initRecordCallback: msg", msg)
    local msg = cjson.decode(msg)
    if msg["success"] then
        fun = "invoke.recorder.start"
        data = {}
        g_game.logicObjProxy:CallClientFun(fun, data, "startRecordCallback", g_game.recorderStartCallObj)
        if g_game.recordFBO == nil then
            g_game.recordFBO = g_game.context:GetColorFrameBufferPool():CreateColorFrameBuffer(SColorBufferKey(g_game.recordWidth, g_game.recordHeight, EPixelFormat.ePF_R8G8B8A8))
        end
    end

end

function recordPhotoAlbumCallback(obj, msg)
    print("recordPhotoAlbumCallback: msg", msg)
    local msg = cjson.decode(msg)
    if g_game.bIsIos then
        if msg and msg["data"] and msg["data"]["status"] == 1 then
            if g_game.onSaveToPhotoAlbum then
                g_game:onSaveToPhotoAlbum(true)
                return
            end
        end
        if g_game.onSaveToPhotoAlbum then
            g_game:onSaveToPhotoAlbum(false)
            return
        end
    else
        if msg and msg["success"] then
            if g_game.onSaveToPhotoAlbum then
                g_game:onSaveToPhotoAlbum(true)
                return
            end
        end
        if g_game.onSaveToPhotoAlbum then
            g_game:onSaveToPhotoAlbum(false)
            return
        end
    end

end

function recordPhotoReleaseCallback(obj, msg)
    print("recordPhotoReleaseCallback: msg", msg)
    local msg = cjson.decode(msg)
end

function recordCompleteCallback(obj, msg)
    print("recordCompleteCallback: msg", g_game.bDiscardRecordVideo, msg)
    local msg = cjson.decode(msg)
    local msgData = msg["data"]
    print("xxxxxx saveToPhotoAlbum", msgData["path"], g_game.bDiscardRecordVideo)
    if msgData["success"] then
        if not g_game.bDiscardRecordVideo then
            local data = {}
            data['file_type'] = 'video'
            data['file_path'] = msgData["path"]
            local param = {}
            param["data"] = cjson.encode(data)
            print("request msg", cjson.encode(param))

            g_game.logicObjProxy:CallClientFun("invoke.js.base.saveToPhotoAlbum", param, "recordPhotoAlbumCallback", g_game.recordPhotoAlbumCallObj)
            print("recordPhotoReleaseCallback: 1")
            g_game.logicObjProxy:CallClientFun("invoke.recorder.release", {}, "recordPhotoReleaseCallback", g_game.recordPhotoReleaseCallObj)
            print("recordPhotoReleaseCallback: 2")
        else
            print("record remove file", msgData["path"])
            os.remove(msgData["path"])
        end
    end
end

function recordProcessCallback(obj, msg)
    print("recordProcessCallback: msg", msg)
end

function startRecordCallback(obj, msg)
    print("startRecordCallback: msg", msg)
    local msg = cjson.decode(msg)
    g_game.bDiscardRecordVideo = false
    g_game.bStartRecordSuccess = false
    if msg["success"] then
        g_game.bStartRecordSuccess = true
        if g_game.registerRecord == nil then
            g_game.registerRecord = true
            g_game.logicObjProxy:CallClientFun("register.recorder.on_complete", {}, "recordCompleteCallback", g_game.recordCompleteCallObj)
            g_game.logicObjProxy:CallClientFun("register.recorder.on_process", {}, "recordProcessCallback", g_game.recordProcessCallObj)
        end
    end

    print("startRecordCallback , 1111")
    g_game:onRecordState()
end



function stopRecordCallback(obj, msg)
    print("stopRecord: msg", msg)
    local msg = cjson.decode(msg)
    if msg and msg["success"] then
        if g_game.onStopRecord then
            g_game:onStopRecord(true)
            return
        end
    end
    if g_game.onStopRecord then
        g_game:onStopRecord(false)
    end
end

function Game:startRecord()
    local fun = "invoke.recorder.init"
    local data = {}
    local save_Name = 'quark_avatar' .. os.date('%Y-%m-%d-%H-%M-%S') .. '.mp4'
    data['save_name'] = save_Name
    if self.context:GetEvnVar("p_global_dir") then
        data['save_path'] = self.context:GetEvnVar("p_global_dir")..'/' .. save_Name
    else
        data['save_path'] = '/sdcard/' .. save_Name
    end
    print("startRecord", data['save_path'])
    data['video_width'] = self.recordWidth
    data['video_height'] = self.recordHeight
    self.logicObjProxy:CallClientFun(fun, data, "initRecordCallback", self.recorderInitCallObj)
end

function SoDownloadState(obj, msg)
    print("Game: js Download, msg", msg)
    local jsonMsg = cjson.decode(msg)
    print("Game: js Download, download_state", jsonMsg['data'] )
    local data = jsonMsg['data']
    print(Download, data)
    if data['download_state'] == 0 then
        print("Download su")
        g_game:DownloadSo()
    else
        g_game.recordIsRead = (data['download_state'] > 0)
        print("Download recordIsRead", g_game.recordIsRead)
    end
end

function SoDownloadCallback(obj, msg)
    print("Game: js Download1, msg", msg)

    local msg = cjson.decode(msg)
    local data = msg['data']
    local bSuccess = data['success']
    if bSuccess then
        g_game.recordIsRead = true
        print("Download recordIsRead", g_game.recordIsRead)
    end
end

function Game:CheckAndDownload(resName) -- 加载
    local fun = "invoke.so_downloader.getDownloadState"
    local param = {}
    param["so_type"] = resName
    g_game.downloadSoName = resName
    print("Download: request",resName)
    g_game.logicObjProxy:CallClientFun(fun, param, "SoDownloadState", self.downloadStateCallObj)
    print("Game: Download end")
end

function OnLoadTriggerCallback(obj, msg)
    print("Download, msg", msg)
end

function Game:DownloadSo() -- 加载
    local fun = "invoke.so_downloader.download"
    local param = {}
    param["so_type"] = g_game.downloadSoName
    print("DownloadSo: request",'g_game.downloadSoName')
    g_game.logicObjProxy:CallClientFun(fun, param, "OnLoadTriggerCallback", self.downloadsoCallObj)
    print("DownloadSo end")
    
    g_game.logicObjProxy:CallClientFun("register.so_downloader.on_complete", {}, "SoDownloadCallback", self.soDownloadCallback)

end



function Game:onRecordState()

end

function Game:onRecordProcess()

end

function Game:onRecordRender()

end

function Game:recordFrame(texture)
    if self.bStartRecordSuccess == false then
        return false
    end

    local vScale = V2f()
    vScale.x, vScale.y = texture:GetWidth() / self.recordWidth, texture:GetHeight() / self.recordHeight
    vScale.y = vScale.y / vScale.x
    vScale.x = 1.0

    local bkFrameBuffer = self.context:GetGpuDevice():GetFrameBufferColor(0)
    local bkDepth = self.context:GetGpuDevice():GetDepthStencil()
    self.context:GetGpuDevice():SetFrameBufferColor(0, self.recordFBO:Get():GetFrameBufferColor())
    self.context:GetGpuDevice():SetDepthStencil(nil)
    local vColor = Color4c()
    vColor.a = 0
    vColor.r = 0
    vColor.g = 0
    vColor.b = 0
    self.context:GetGpuDevice():ClearBuffer(2, vColor, 1.0, 0 )
    self.render:RenderTex(texture, true, false, vScale, M44f())
    self:onRecordRender()
    self.context:GetGpuDevice():SetFrameBufferColor(0, bkFrameBuffer)
    self.context:GetGpuDevice():SetDepthStencil(bkDepth)
    self.context:SetRecordFrameBuffer(self.recordFBO:Get())
    return true
end

function Game:stopRecord(discard)
    self.bStartRecordSuccess = false
     self.bDiscardRecordVideo = false
    if discard ~= nil then
        self.bDiscardRecordVideo = discard
    end
    local fun = "invoke.recorder.stop"
    local data = {}
    self.logicObjProxy:CallClientFun(fun, data, "stopRecordCallback", self.recorderStopCallObj)
end

function Game:createLogicObj(moduleName, _engineObj, _parent)
    local obj =  {logicObj = nil, engineObj = _engineObj}
    self.objTable[_engineObj:GetAddress()] = obj
    if string.len(moduleName) > 0 then
        local logicObjClass = require(moduleName)
        print("Game: new module before:", moduleName)
        obj.logicObj = logicObjClass:new(_engineObj)
        print("Game: module name:", moduleName)
    end

    if _parent then
        parentLogicObj = self.objTable[_parent:GetAddress()].logicObj
        if parentLogicObj then
            parentLogicObj.logicObjs[_engineObj:GetAddress()] = obj
        end
    end
end

function Game:removeLogicObj(logicObj)
    self.objTable[logicObj.engineObj:GetAddress()] = nil
end

function Game:removeLogicObjFromAddress(address)
    self.objTable[address] = nil
end

function Game:findLogicObj(engineObj)
    return self.objTable[engineObj:GetAddress()]
end

function Game:OnUpdateViewProj(vm, pm)
    g_game.scene.engineObj:GetCamera():SetViewProjMatrix(vm, pm)
end

function Game:onGameBackground()
end

function Game:onGameForeground()
end

function Game:onGameGoout()
end

function Game:onGameBack()
end

function OnErrorHandler( err )
    print( "ERROR:", err )
end

function _gameRender(time, frameBuffer)
    return g_game:gameRender(time, frameBuffer)
end

function gameRender(time, frameBuffer)
    local curFrameBuffer = frameBuffer
	g_game.screenWidth, g_game.screenHeight = frameBuffer:Get():GetFrameBufferColor():GetWidth(), frameBuffer:Get():GetFrameBufferColor():GetHeight()
    local status, retval = xpcall( _gameRender, OnErrorHandler, time, frameBuffer )
    if status == true then
		return retval
    else
		return curFrameBuffer
	end
end

function _gameCreateLogicObj(moduleName, _engineObj, _parent)
    if g_game == nil then
        g_game = Game:new(nil)
        print("create nil game")
    end

    return g_game:createLogicObj(moduleName, _engineObj, _parent)
end

function gameCreateLogicObj(moduleName, _engineObj, _parent)
    local status, retval = xpcall( _gameCreateLogicObj, OnErrorHandler, moduleName, _engineObj, _parent )
    return status
end

function _gameOnTouchEvent(time, touchEvent)
    local inputEvent = {
        x = touchEvent.fX,
        y = touchEvent.fY,
        fingerId = touchEvent.fingerId,
        state = touchEvent.eTouchEvent,
        time = time,
    }
    -- print("input event", inputEvent.x, inputEvent.y, inputEvent.fingerId, inputEvent.state, inputEvent.time)
    TouchGestureEngine:OnTouch(inputEvent)
end

function gameOnTouchEvent(time, touchEvent)
    -- print("HCD: todo xcall _gameOnTouchEvent")
    xpcall( _gameOnTouchEvent, OnErrorHandler, time, touchEvent )
end

function _gameDeleteLogicObj(_engineObj)
    return g_game:removeLogicObjFromAddress(_engineObj:GetAddress())
end

function gameDeleteLogicObj( _engineObj)
    local status, retval = xpcall( _gameDeleteLogicObj, OnErrorHandler, _engineObj )
    return status
end


function _onGameBackground()
    return g_game:onGameBackground()
end

function _onGameForeground()
    return g_game:onGameForeground()
end

function onGameBackground()
    local status, retval = xpcall( _onGameBackground, OnErrorHandler )
    return status
end

function onGameForeground()
    local status, retval = xpcall( _onGameForeground, OnErrorHandler )
    return status
end


function _onGameGoout()
    return g_game:onGameGoout()
end

function _onGameBack()
    return g_game:onGameBack()
end

function onGameGoout()
    local status, retval = xpcall( _onGameGoout, OnErrorHandler )
    return status
end

function onGameBack()
    local status, retval = xpcall( _onGameBack, OnErrorHandler )
    return status
end

function _gameDestroy()
    g_game = nil
end

function gameDestroy()
    local status, retval = xpcall( _gameDestroy, OnErrorHandler )
    return status
end


