luainfo = {}
luainfo["type"] = "CLuaEffectUnit"
luainfo["Name"] = "Test"

luainfoint = {}
luainfoint["version"] = 20200
luainfoint["linktype"] = EEffectLinkType.eELT_Screen
luainfoint["detectType"] = 0

moveParam = 0.0
bgEffectPath = nil

segVert = [[
precision highp float;

attribute vec4 aVertexCo;
attribute vec2 aTextureCo;

uniform float offsetX;
uniform float offsetY;

varying vec2 vTextureCo;
varying vec2 vTextureCo1;

void main(){
    gl_Position = aVertexCo;
    vTextureCo = aTextureCo;
    vTextureCo1 = vec2((aTextureCo.x - 0.5) * offsetX + 0.5, (aTextureCo.y - 0.5) * offsetY + 0.5) ;
}
]]

segFrag = [[
precision highp float;
varying vec2 vTextureCo;
varying vec2 vTextureCo1;
//背景
uniform sampler2D uTexture;
//前景
uniform sampler2D uTexture1;
//mask
uniform sampler2D uTexture2;
//smooth起始值、结束值及描边权重
uniform float start;
uniform float end;

float inSmoothStep(float start, float end, float dt){
    float t = clamp((dt - start) / (end - start), 0.0, 1.0);
    return t * t * (3.0 - 2.0 * t);
}

void main() {
    vec4 fore = texture2D( uTexture,  vTextureCo);
    vec4 back = texture2D( uTexture1, vTextureCo1);

    vec4 backMix = back.a * back + (1.0 - back.a) * fore;

    // float mask = smoothstep(start,end,texture2D( uTexture2, vTextureCo).r);
    float mask = inSmoothStep(start,end,texture2D( uTexture2, vTextureCo).r);
    vec4 color = mix(backMix,fore,mask);
    gl_FragColor = color;
    // if(mask != 0.0 && mask != 1.0){
    //     gl_FragColor = vec4(1.0,0.0,0.0,1.0);
    // }else{
    //     gl_FragColor = color;
    // }
}
]]

smoothVert = [[
precision highp float;

attribute vec4 aVertexCo;
attribute vec2 aTextureCo;

varying vec2 vTextureCo;

void main(){
    gl_Position = aVertexCo;
    vTextureCo = aTextureCo;
}
]]

smoothFrag = [[
precision highp float;
varying vec2 vTextureCo;
//背景
uniform sampler2D uTexture;
uniform float uWidthStep;
uniform float uHeightStep;

void main() {
    vec4 center = texture2D(uTexture, vTextureCo);
    center += texture2D(uTexture, vTextureCo + vec2(uWidthStep, uHeightStep));
    center += texture2D(uTexture, vTextureCo + vec2(-uWidthStep, uHeightStep));
    center += texture2D(uTexture, vTextureCo + vec2(uWidthStep, -uHeightStep));
    center += texture2D(uTexture, vTextureCo + vec2(uWidthStep, 0.0));
    center += texture2D(uTexture, vTextureCo + vec2(0.0, uHeightStep));
    center += texture2D(uTexture, vTextureCo + vec2(-uWidthStep, 0.0));
    center += texture2D(uTexture, vTextureCo + vec2(0.0, -uHeightStep));
    center += texture2D(uTexture, vTextureCo + vec2(-uWidthStep, -uHeightStep));

    gl_FragColor = center / 9.0;
}
]]

lastDetectType = 0

function Init(effectUnit)

    effectUnit:RegFloatProperty("smoothStep", 1.57)
    effectUnit:RegFloatProperty("borderStart", 0.0)
    effectUnit:RegFloatProperty("borderEnd", 1.0)

    context = effectUnit:GetEngineContext()
    resObjMgr = context:GetResObjMgr()
    effectMgr = context:GetEffectMgr()
    render = context:GetRenderer()

    materialSeg = render:CreateMaterialFromCode(segVert, segFrag)
    materialSeg:Get():SetTransparentState(ETransparentState.eTS_BlendRZ)
    paramTex1 = SParamName("uTexture")
    paramTex2 = SParamName("uTexture1")
    paramTex3 = SParamName("uTexture2")
    paramOffsetX = SParamName("offsetX")
    paramOffsetY = SParamName("offsetY")
    paramStart = SParamName("start")
    paramEnd = SParamName("end")

    materialSmooth = render:CreateMaterialFromCode(smoothVert, smoothFrag)
    materialSmooth:Get():SetTransparentState(ETransparentState.eTS_BlendRZ)
    paramTexSmooth = SParamName("uTexture")
    paramUWidthStep = SParamName("uWidthStep")
    paramUHeightStep = SParamName("uHeightStep")


    clearColor = Color4c()
    clearColor.r = 0
    clearColor.g = 0
    clearColor.b = 0
    clearColor.a = 0
end

function Reset(effectUnit)
-- body
end

function OnErrorHandler(err)
    print("Error : ", err)
end

function OnRender(effectUnit, frameBufferTex, pLinkInfo)
    local status, retval = xpcall(ProtectOnRender, OnErrorHandler, effectUnit, frameBufferTex, pLinkInfo)
    if status == true then
        return retval
    else
        return frameBufferTex
    end
end

function updateDetectTypeToZero(effectUnit)
    if lastDetectType ~= 0 then
        effectUnit:UpdateDetectType(0)
        lastDetectType = 0
    end
end

function ProtectOnRender(effectUnit, frameBufferTex, pLinkInfo)
    local isEnableUserBg = tonumber(context:GetEvnVar("KeyEnableBackground"))
    if isEnableUserBg == 0 then
        updateDetectTypeToZero(effectUnit)
        return frameBufferTex
    end
    local tempUserBgPath = context:GetEvnVar("UserBackground")
    if bgEffectPath ~= tempUserBgPath then
        bgEffectPath = tempUserBgPath
        if bgEffectPath ~= nil and bgEffectPath ~= "" then
            res0 = resObjMgr:CreateResObj(bgEffectPath, "", false)
            if res0 == nil or res0:Get() == nil then
                updateDetectTypeToZero(effectUnit)
                return frameBufferTex
            end
            bgMediaTex = TypeCast(res0:Get(), "CTexture")
        else
            res0 = nil
            bgMediaTex = nil
        end
    end
    if bgMediaTex == nil or bgMediaTex:GetGpuTexture() == nil then
        updateDetectTypeToZero(effectUnit)
        return frameBufferTex
    end
    if lastDetectType ~= EDetectType.eDT_Segment then
        effectUnit:UpdateDetectType(EDetectType.eDT_Segment)
        lastDetectType = EDetectType.eDT_Segment
    end
    local segmentTex = effectMgr:GetSegmentTex()
    if segmentTex == nil then
        return frameBufferTex
    end

	local width, height = frameBufferTex:Get():GetFrameBufferColor():GetWidth(), frameBufferTex:Get():GetFrameBufferColor():GetHeight()
    local mWorld = M44f()
    local tempBuffer = context:GetColorFrameBufferPool():CreateColorFrameBuffer(SColorBufferKey(width, height, EPixelFormat.ePF_R8G8B8A8))
    local smoothBuffer = context:GetColorFrameBufferPool():CreateColorFrameBuffer(SColorBufferKey(
    	segmentTex:GetWidth(), segmentTex:GetHeight(), EPixelFormat.ePF_R8G8B8A8))

    context:GetGpuDevice():SetFrameBufferColor(0, smoothBuffer:Get():GetFrameBufferColor())
    context:GetGpuDevice():ClearBuffer(2, clearColor, 0.0, 0)
    materialSmooth:Get():SetGpuTex2DParam(paramTexSmooth, segmentTex)
    materialSmooth:Get():SetFloat1Param(paramUWidthStep, effectUnit:FindFloat("smoothStep") / segmentTex:GetWidth())
    materialSmooth:Get():SetFloat1Param(paramUHeightStep, effectUnit:FindFloat("smoothStep") / segmentTex:GetHeight())
    render:RenderQuad(materialSmooth:Get(), mWorld, effectUnit:GetSimulateTime())

    local bgWidth, bgHeight = bgMediaTex:GetWidth(), bgMediaTex:GetHeight()
    local bgRate = bgWidth / bgHeight
    local inRate = width / height
    local offsetX = 0.0
    local offsetY = 0.0
    if bgRate > inRate then
    	offsetX = inRate / bgRate
   		offsetY = 1.0
   	else
   		offsetY = bgRate / inRate
   		offsetX = 1.0
    end

    context:GetGpuDevice():SetFrameBufferColor(0, tempBuffer:Get():GetFrameBufferColor())
    context:GetGpuDevice():ClearBuffer(2, clearColor, 0.0, 0)
    materialSeg:Get():SetGpuTex2DParam(paramTex1, frameBufferTex:Get():GetFrameBufferColor())
    materialSeg:Get():SetGpuTex2DParam(paramTex2, bgMediaTex:GetGpuTexture())
    materialSeg:Get():SetGpuTex2DParam(paramTex3, smoothBuffer:Get():GetFrameBufferColor())
    materialSeg:Get():SetFloat1Param(paramStart, effectUnit:FindFloat("borderStart"))
    materialSeg:Get():SetFloat1Param(paramEnd, effectUnit:FindFloat("borderEnd"))
    materialSeg:Get():SetFloat1Param(paramOffsetX, offsetX)
    materialSeg:Get():SetFloat1Param(paramOffsetY, offsetY)
    render:RenderQuad(materialSeg:Get(), mWorld, effectUnit:GetSimulateTime())

    return tempBuffer

end
