local BaseGestureDetector = require "touch_gesture_engine/gesture_detector/base_gesture_detector"
local MiscUtil = require "utils/misc_util"
local MathUtil = require "utils/math_util"

local PinchGestureDetector = {
    state = {},
    distance = nil, -- two finger
    center = nil, -- two finger
    lastTime = nil,
    lastFingerEvent = nil,
    -- latestFingerEvent = {},
}

function PinchGestureDetector:Create()
    return BaseGestureDetector.BaseGestureDetectorInherit(self)
end

-- virtual function
function PinchGestureDetector:Init(config, listener)
    self.listener = listener
    print("PinchGestureDetector:Init")
    return true
end

local THRESHOLD_TAP_TIME = 0.3 * 1000000
local THRESHOLD_TAP_DISTANCE = 0.1

local function TableLen(t)
    local len = 0
    for k, v in pairs(t) do
        len = len + 1
    end
    return len
end

local function CalFingerDistance(state)
    print("PinchGestureDetector: in CalDistance")
    local vecEvent = {}
    for k, v in pairs(state) do
        table.insert(vecEvent, v)
    end

    if TableLen(vecEvent) ~= 2 then
        print("PinchGestureDetector: event num != 2")
        return 0.0
    end

    local x1, y1 = vecEvent[1].x, vecEvent[1].y
    local x2, y2 = vecEvent[2].x, vecEvent[2].y
    local distance = math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2))
    print("PinchGestureDetector: CalDistance:", x1, y1, x2, y2, distance)
    return distance
end

local function CalCenter(a, b)
    return {(a.x + b.x) / 2, (a.y + b.y) / 2}
end

local function CalDistance(a, b)
    local x, y = a[1] - b[1], a[2] - b[2]
    return math.sqrt(x * x + y * y)
end

local PINCH_MAX_CENTER_DELTA_STEP = 0.1
local function CalCenterDelta(detector)
    local vecEvent = {}
    for k, v in pairs(detector.state) do
        table.insert(vecEvent, v)
    end

    if TableLen(vecEvent) ~= 2 then
        print("PinchGestureDetector: event num != 2")
        return 0
    end

    local curCenter = CalCenter(vecEvent[1], vecEvent[2])
    if not detector.center then
        detector.center = curCenter
        return 0
    end

    local lastCenter = detector.center
    detector.center = curCenter
    local delta = {curCenter[1] - lastCenter[1], curCenter[2] - lastCenter[2]} -- CalDistance(curCenter, detector.center)

    return delta
end

local PINCH_MAX_DELTA = 0.05
local PINCH_DOUBLE_FINGER_MOVE_TIME_INTERVAL = 1 / 30 * 1000
local function GenOutputEvent(detector, time)
    local oe = {
        scaleDelta = 0,
        centerDelta = {
            xDelta = 0,
            yDelta = 0
        },
    }

    if not detector.lastTime then
        detector.lastTime = time
    end

    local lastDistance = detector.distance
    if lastDistance == nil then
        detector.distance = CalFingerDistance(detector.state)
        print("PinchGestureDetector: lastDistance is nil")
        return oe
    end

    detector.distance = CalFingerDistance(detector.state)
    oe.scaleDelta = detector.distance - lastDistance

    if time - detector.lastTime > PINCH_DOUBLE_FINGER_MOVE_TIME_INTERVAL then
        local lastFingerEvent = detector.lastFingerEvent
        local latestFingerEvent = detector.state
        if not lastFingerEvent then
            detector.lastFingerEvent = MiscUtil.DeepCopy(detector.state)
            lastFingerEvent = detector.lastFingerEvent
        end

        local delta = {}
        for k, last in pairs(lastFingerEvent) do
            local latest = latestFingerEvent[k]
            if latest then
                table.insert(delta, {latest.x - last.x, latest.y - last.y})
            end
        end

        MiscUtil.PrintTable(delta)
        if TableLen(delta) ~= 2 then
            print("PinchGestureDetector: not 2 finger") 
        else 
            local deltaL, deltaR = delta[1], delta[2]
            if not (deltaL[1] == 0 and deltaL[2] == 0) and not (deltaR[1] == 0 and deltaR[2] == 0) then
                local cos = MathUtil.Dot(deltaL, deltaR)
                print("PinchGestureDetector: cos", cos) 
                if cos > 0.8 and cos <= 1 then
                    oe.centerDelta.xDelta = deltaL[1]
                    oe.centerDelta.yDelta = deltaL[2]
                end

                detector.lastFingerEvent = MiscUtil.DeepCopy(latestFingerEvent)
                detector.lastTime = time
            else
                oe.scaleDelta = 0
            end
        end
    end

    if (oe.scaleDelta > PINCH_MAX_DELTA) then
        print("PinchGestureDetector: too large delta", oe.scaleDelta)
        oe.scaleDelta = 0.0
    end

    if oe.centerDelta.xDelta > PINCH_MAX_DELTA or oe.centerDelta.yDelta > PINCH_MAX_DELTA then
        print("PinchGestureDetector: too large center delta", oe.centerDelta.xDelta, oe.centerDelta.yDelta)
        oe.centerDelta.xDelta = 0
        oe.centerDelta.yDelta = 0
    end

    print("PinchGestureDetector: scaleDelta, center_delta", oe.scaleDelta, oe.centerDelta.xDelta, oe.centerDelta.yDelta)
    return oe
end

function PinchGestureDetector:Reset()
    self.gestureInProgress = false 
    self.state = {}
    self.distance = nil 
    self.center = nil
    self.lastFingerEvent = nil
end

function PinchGestureDetector:AllUp()
    return TableLen(self.state) == 0
    -- for k, v in pairs(self.state) do
    --     print("PinchGestureDetector: finger id", v.fingerId, "state", v.state)
    --     if v.state ~= 2 and v.state ~= 3 then
    --         return false
    --     end
    -- end
    -- return true
end

-- virtual function
-- state : down-0, move-1, up-2, cancle-3
function PinchGestureDetector:HandleStartProgressEvent(gestureInputEvent) 
    print("PinchGestureDetector HandleStartProgressEvent")
    print("PinchGestureDetector fingerNum", TableLen(self.state))
    -- MiscUtil.PrintTable(self.state)

    local state = gestureInputEvent.state
    local time = gestureInputEvent.time
    local fingerId = gestureInputEvent.fingerId

    if state == 0 then
        self.state[fingerId] = gestureInputEvent
        print("PinchGestureDetector: get down")

    elseif state == 2 or state == 3 then
        self.state[fingerId] = nil
        -- self.state = {}
        -- self.distance = nil 
        -- self.center = nil
        print("PinchGestureDetector: get up or cancle, ", state)

    elseif state == 1 then
        self.state[fingerId] = gestureInputEvent
        print("PinchGestureDetector: add finger, fingerNum", TableLen(self.state))
        -- MiscUtil.PrintTable(self.state)

        local fingerNum = TableLen(self.state)
        if fingerNum == 2 then
            self.gestureInProgress = self.listener:OnPinchBegin(GenOutputEvent(self, time))
        end
    end

    if self:AllUp() then
        print("PinchGestureDetector: all up")
        self:Reset()
    end
end

-- virtual function
function PinchGestureDetector:HandleInProgressEvent(gestureInputEvent) 
    print("PinchGestureDetector HandleInProgressEvent")
    -- MiscUtil.PrintTable(self.state)

    local state = gestureInputEvent.state
    local fingerId = gestureInputEvent.fingerId
    local time = gestureInputEvent.time
    if state == 0 then
        self.state[fingerId] = gestureInputEvent
        print("PinchGestureDetector: get down")

    elseif state == 2 or state == 3 then
        self.state[fingerId] = nil
        self.listener:OnPinchEnd(GenOutputEvent(self, time))
        self.gestureInProgress = false 
        -- self.distance = nil 
        -- self.center = nil
        -- self.state = {}
        print("PinchGestureDetector: get up or cancle, ", state)

    elseif state == 1 then
        self.state[fingerId] = gestureInputEvent
        local fingerNum = TableLen(self.state)

        if fingerNum == 2 then
            self.gestureInProgress = self.listener:OnPinch(GenOutputEvent(self, time))
        else
            print("PinchGestureDetector: fingerNum ~= 2")
        end

    end

    if self:AllUp() then
        print("PinchGestureDetector: all up")
        self:Reset()
    end
end

return PinchGestureDetector

