local BaseGestureDetector = require "touch_gesture_engine/gesture_detector/base_gesture_detector"

local SwipeGestureDetector = {
    _moveStateCnt = 0
}

function SwipeGestureDetector:Create()
    return BaseGestureDetector.BaseGestureDetectorInherit(self)
end

-- virtual function
function SwipeGestureDetector:Init(config, listener)
    self.listener = listener
    print("SwipeGestureDetector:Init")
    return true
end

local MAX_SWIPE_STEP = 0.1
local function GenOutputEvent(swipeGestureDetector)
    local oe = {
        x = swipeGestureDetector.currEvent.x,
        y = swipeGestureDetector.currEvent.y
    }

    oe.deltaX = 0
    oe.deltaY = 0
    if swipeGestureDetector.prevEvent then
        oe.deltaX = swipeGestureDetector.currEvent.x - swipeGestureDetector.prevEvent.x
        oe.deltaY = swipeGestureDetector.currEvent.y - swipeGestureDetector.prevEvent.y
    else
        print("SwipeGestureDetector: pre event is NULL")
    end

    if math.abs(oe.deltaX) > MAX_SWIPE_STEP or math.abs(oe.deltaY) > MAX_SWIPE_STEP then
        if oe.deltaX > 0 then
            oe.deltaX = MAX_SWIPE_STEP
        else
            oe.deltaX = -MAX_SWIPE_STEP
        end

        if oe.deltaY > 0 then
            oe.deltaY = MAX_SWIPE_STEP
        else
            oe.deltaY = -MAX_SWIPE_STEP
        end
    end

    return oe
end

-- virtual function
-- state : down-0, move-1, up-2, cancle-3
function SwipeGestureDetector:HandleStartProgressEvent(gestureInputEvent) 
    local state = gestureInputEvent.state
    print("SwipeGestureDetector: HandleStartProgressEvent", state, gestureInputEvent.fingerId)

    if state == 0 then
        self._moveStateCnt = 0
        if self.prevEvent then
            local time = gestureInputEvent.time - self.prevEvent.time
            print("time", time)
            if time > 1.0 * 1000 then
                print("SwipeGestureDetector: too long interval time, to ResetState")
                self:ResetState()
            end
        end
        self:UpdateStateByEvent(gestureInputEvent)
        -- self:ResetState()
        print("SwipeGestureDetector: get down")
        return
    end

    self:UpdateStateByEvent(gestureInputEvent)
    if state == 1 then
        self._moveStateCnt = self._moveStateCnt + 1
        if self._moveStateCnt >= 3 then
            print("SwipeGestureDetector: get move")
            if self:NotSingle() then
                print("SwipeGestureDetector: not single finger")
                -- self:ResetState()
                return
            end
            self.gestureInProgress = self.listener:OnSwipeBegin(GenOutputEvent(self))
            print("SwipeGestureDetector: do move")
        end
        return
    end

    if state == 2 then
        self._moveStateCnt = 0
    end

    if self:AllUp() then
        print("SwipeGestureDetector: all up")
        self._moveStateCnt = 0
        self:ResetState()
    end
end

-- virtual function
function SwipeGestureDetector:HandleInProgressEvent(gestureInputEvent) 
    local state = gestureInputEvent.state
    print("SwipeGestureDetector HandleInProgressEvent", state, gestureInputEvent.fingerId)
    self:UpdateStateByEvent(gestureInputEvent)
    if self:NotSingle() then
        print("SwipeGestureDetector: not single finger")
        self.gestureInProgress = false
        self._moveStateCnt = 0
        self.listener:OnSwipeEnd(GenOutputEvent(self))
        -- self:ResetState()
        return
    end

    if state == 2 or state == 3 then
        self.listener:OnSwipeEnd(GenOutputEvent(self))
        self._moveStateCnt = 0
        self:ResetState()
        return
    end

    if state == 1 then
        print("SwipeGestureDetector: to OnSwipe")
        self.listener:OnSwipe(GenOutputEvent(self))
        return
    end
end

return SwipeGestureDetector

