local json = require "utils/json"
local FileUtil = require "utils/file_util"
local MiscUtil = require "utils/misc_util"
local GestureDetectorFactory = require "touch_gesture_engine/gesture_detector/gesture_detector_factory"

local TouchGestureEngine = {
    -- config   = nil,
    -- listener = nil,
    detectors = {}
}

function TouchGestureEngine:RegListener(config, listener)
    -- conf = json.decode(FileUtil.ReadFile(config))
    print("TouchGestureEngine: in RegListener")
    conf = json.decode(config)
    for k, v in ipairs(conf) do
        -- MiscUtil.PrintTable(v)
        detectorName = v.detector
        assert(type(detectorName) == "string", "no filed[detector] in gesture engine config")
        detector = GestureDetectorFactory:GetDetector(detectorName)
        assert(detector, ("invalid gesture detector name:" .. detectorName))
        assert(detector:Init(v.config, listener), "gesture detector Init fail, name:", detectorName)
        table.insert(self.detectors, detector)
        print("detector init success, name:", detectorName)
    end
    
    print("TouchGestureEngine: RegListener success")
end

function TouchGestureEngine:RegDetector(detector, listener)
    detector.listener = listener
    table.insert(self.detectors, detector)
end

-- state : down-0, move-1, up-2, cancle-3
function TouchGestureEngine:OnTouch(gestureInputEvent)
    -- print("TouchGestureEngine: get gesture input event:")
    local fun = g_game.OnGestureTouchBegin
    if fun then 
        fun()
    end

    -- MiscUtil.PrintTable(gestureInputEvent)
    for i, d in ipairs(self.detectors) do
        d:OnTouch(gestureInputEvent)
    end

    fun = g_game.OnGestureTouchEnd
    if fun then 
        fun()
    end
end

return TouchGestureEngine