local MathUtil = {}

function MathUtil.Normalize(vec)
    local sum = 0.0
    for i, v in pairs(vec) do
        sum = sum + v * v
    end
    local length = math.sqrt(sum)
    local ret = {}
    if length ~= 0.0 then
        for i, v in pairs(vec) do
            table.insert(ret, v / length)
        end
    end
    return ret
end

function MathUtil.Cross(a, b)
    return {a[2]*b[3]-b[2]*a[3], b[1]*a[3]-a[1]*b[3], a[1]*b[2]-b[1]*a[2]}
end

function MathUtil.Dot(a, b)
    local l, r = MathUtil.Normalize(a), MathUtil.Normalize(b)
    return l[1] * r[1] + l[2] * r[2]
end

function MathUtil.V3fCos(a, b)
    local l, r = MathUtil.V3fNormalize(a), MathUtil.V3fNormalize(b)
    local ret = l.x * r.x + l.y * r.y + l.z * r.z
    return math.min(math.max(ret, -1), 1)
end

function MathUtil.TableMerge(a, b)
    for i, v in pairs(b) do
        table.insert(a, v)
    end
    return a
end

function MathUtil.Distance(a, b)
    local x, y, z = a[1] - b[1], a[2] - b[2], a[3] - b[3]
    return math.sqrt(x * x + y * y + z * z)
end

function MathUtil.Lerp(t1, t2, progress)
    local ret = {}
    for i = 1, #t1 do
        table.insert(ret, t1[i] + (t2[i] - t1[i]) * progress)
    end
    return ret
end

function MathUtil.V3fSub(v1, v2)
    return V3f(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z)
end

function MathUtil.V3fDiv(v1, n)
    return V3f(v1.x / n, v1.y / n, v1.z / n)
end

function MathUtil.V3fLerp(v1, v2, progress)
    return V3f(v1.x + (v2.x - v1.x) * progress, v1.y + (v2.y - v1.y) * progress, v1.z + (v2.z - v1.z) * progress)
end

function MathUtil.V3fLength(d)
    return math.sqrt(d.x * d.x + d.y * d.y + d.z * d.z)
end

function MathUtil.V3fDistance(a, b)
    local x, y, z = a.x - b.x, a.y - b.y, a.z - b.z
    return math.sqrt(x * x + y * y + z * z)
end

function MathUtil.V3fCopy(v)
    local ret = V3f(v.x, v.y, v.z)
    return ret
end

function MathUtil.V3fNormalize(v)
    local sum = v.x * v.x + v.y * v.y + v.z * v.z
    local length = math.sqrt(sum)
    if length == 0 then
        return V3f(0, 0, 0)
    end
    return V3f(v.x / length, v.y / length, v.z / length)
end

function MathUtil.EulerLerp(v1, v2, progress)
    local e1 = Eulerf(v1.x / 180 * 3.1415926, v1.y / 180 * 3.1415926, v1.z / 180 * 3.1415926, 257)
    local e2 = Eulerf(v2.x / 180 * 3.1415926, v2.y / 180 * 3.1415926, v2.z / 180 * 3.1415926, 257)
    local q1, q2 = e1:toQuat(), e2:toQuat()
    local newQ = slerpShortestArc(q1:normalize(), q2:normalize(), progress)
    local ret = Eulerf(0, 0, 0, 257)
    ret:extractQuat(newQ)
    ret = ret:toXYZVector()
    ret.x, ret.y, ret.z = ret.x / 3.1415926 * 180, ret.y / 3.1415926 * 180, ret.z / 3.1415926 * 180
    return ret
end

function MathUtil.M44fCopy(m)
    local ret = M44f()
    ret.m11, ret.m12, ret.m13, ret.m14 = m.m11, m.m12, m.m13, m.m14
    ret.m21, ret.m22, ret.m23, ret.m24 = m.m21, m.m22, m.m23, m.m24
    ret.m31, ret.m32, ret.m33, ret.m34 = m.m31, m.m32, m.m33, m.m34
    ret.m41, ret.m42, ret.m43, ret.m44 = m.m41, m.m42, m.m43, m.m44
    return ret
end

return MathUtil